@@ -216,6 +216,7 @@ PGrnGetEncoding(void)
 	case PG_EUC_JIS_2004:
 		return GRN_ENC_EUC_JP;
 	case PG_LATIN1:
+	case PG_WIN1252:
 		return GRN_ENC_LATIN1;
 	case PG_KOI8R:
 		return GRN_ENC_KOI8R;
@@ -834,6 +835,9 @@ PGrnCreate(Relation index,
 		attribute = data.desc->attrs[data.i];
 		if (PGrnAttributeIsJSONB(attribute->atttypid))
 		{
+			data.forFullTextSearch = false;
+			data.forRegexpSearch = false;
+			data.forPrefixSearch = false;
 			PGrnJSONBCreate(&data);
 		}
 		else
@@ -1037,6 +1041,8 @@ PGrnCollectScoreScanOpaque(Relation table, HeapTuple tuple, PGrnScanOpaque so)
 		grn_obj_unlink(ctx, tableCursor);
 	}
 
+	grn_obj_close(ctx, records);
+
 	return score;
 }
 
@@ -1783,7 +1789,7 @@ pgroonga_match_contain_text(PG_FUNCTION_ARGS)
 {
 	text *target = PG_GETARG_TEXT_PP(0);
 	ArrayType *keywords = PG_GETARG_ARRAYTYPE_P(1);
-	grn_bool matched;
+	grn_bool matched = GRN_FALSE;
 	int i, n;
 
 	n = ARR_DIMS(keywords)[0];
@@ -1817,7 +1823,7 @@ pgroonga_query_contain_text(PG_FUNCTION_ARGS)
 {
 	text *target = PG_GETARG_TEXT_PP(0);
 	ArrayType *queries = PG_GETARG_ARRAYTYPE_P(1);
-	grn_bool matched;
+	grn_bool matched = GRN_FALSE;
 	int i, n;
 
 	n = ARR_DIMS(queries)[0];
@@ -2157,12 +2163,12 @@ PGrnScanOpaqueReinit(PGrnScanOpaque so)
 	GRN_OBJ_FIN(ctx, &(so->maxBorderValue));
 	if (so->sorted)
 	{
-		grn_obj_unlink(ctx, so->sorted);
+		grn_obj_close(ctx, so->sorted);
 		so->sorted = NULL;
 	}
 	if (so->searched)
 	{
-		grn_obj_unlink(ctx, so->searched);
+		grn_obj_close(ctx, so->searched);
 		so->searched = NULL;
 	}
 }
@@ -3270,6 +3276,26 @@ PGrnEnsureCursorOpened(IndexScanDesc scan,
 	}
 }
 
+static grn_id
+PGrnScanOpaqueResolveID(PGrnScanOpaque so)
+{
+	grn_id recordID = so->currentID;
+
+	if (so->sorted)
+	{
+		GRN_BULK_REWIND(&(buffers->general));
+		grn_obj_get_value(ctx, so->sorted, recordID, &(buffers->general));
+		recordID = GRN_RECORD_VALUE(&(buffers->general));
+	}
+	if (so->searched)
+	{
+		grn_table_get_key(ctx, so->searched, recordID,
+						  &recordID, sizeof(grn_id));
+	}
+
+	return recordID;
+}
+
 #ifdef PGRN_SUPPORT_INDEX_ONLY_SCAN
 static void
 PGrnGetTupleFillIndexTuple(PGrnScanOpaque so,
@@ -3287,18 +3313,7 @@ PGrnGetTupleFillIndexTuple(PGrnScanOpaque so,
 	values = palloc(sizeof(Datum) * desc->natts);
 	isNulls = palloc(sizeof(bool) * desc->natts);
 
-	recordID = so->currentID;
-	if (so->sorted)
-	{
-		GRN_BULK_REWIND(&(buffers->general));
-		grn_obj_get_value(ctx, so->sorted, recordID, &(buffers->general));
-		recordID = GRN_RECORD_VALUE(&(buffers->general));
-	}
-	if (so->searched)
-	{
-		grn_table_get_key(ctx, so->searched, recordID,
-						  &recordID, sizeof(grn_id));
-	}
+	recordID = PGrnScanOpaqueResolveID(so);
 
 	for (i = 0; i < desc->natts; i++)
 	{
@@ -3334,7 +3349,10 @@ pgroonga_gettuple_raw(IndexScanDesc scan,
 
 	if (scan->kill_prior_tuple && so->currentID != GRN_ID_NIL)
 	{
-		grn_table_delete_by_id(ctx, so->sourcesTable, so->currentID);
+		grn_id recordID;
+
+		recordID = PGrnScanOpaqueResolveID(so);
+		grn_table_delete_by_id(ctx, so->sourcesTable, recordID);
 	}
 
 	if (so->indexCursor)
